@@ -1,14 +1,53 @@
-import re
-import warnings
+"""
+serverly - http.server wrapper and helper
+--
+
+
+Attributes
+--
+`address: tuple = ('localhost', 8080)` The address used to register the server. Needs to be set before running start()
+
+`name: str = 'PyServer'` The name of the server. Used for logging purposes only.
+
+`logger: fileloghelper.Logger = Logger()` The logger used for logging (surprise!!). See the docs of fileloghelper for reference.
+
+
+Methods
+--
+`static_page(file_path, path)` register a static page while the file is located under `file_path` and will serve `path`
+
+`register_get(func, path: str)` register dynamic GET page (function)
+
+`register_post(func, path: str)` register dynamic POST page (function)
+
+`unregister(method: str, path: str)`unregister any page (static or dynamic). Only affect the `method`-path (GET / POST)
+
+`start(superpath: str="/")` start the server after applying all relevant attributes like address. `superpath` will replace every occurence of SUPERPATH/ or /SUPERPATH/ with `superpath`. Especially useful for servers orchestrating other servers.
+
+
+Decorators (technically methods)
+--
+`serves_get(path: str`/`serves_post(path: str)` Register the function to serve a specific path.
+Example:
+```
+@serves_get("/hello(world)?")
+def hello_world(data):
+    return {"response_code": 200, "Content-type": "text/plain"}, "Hello world!"
+```
+This will return "Hello World!" with a status code of 200, as plain text to the client
+"""
 import importlib
+import re
 import urllib.parse as parse
+import warnings
 from functools import wraps
 from http.server import BaseHTTPRequestHandler, HTTPServer
-from serverly.utils import *
 
 from fileloghelper import Logger
+from serverly import default_sites
+from serverly.utils import *
 
-version = "0.0.12"
+version = "0.0.14"
 description = "A really simple-to-use HTTP-server"
 address = ("localhost", 8080)
 name = "PyServer"
@@ -99,7 +138,8 @@ class Server:
     def run(self):
         try:
             logger.set_context("startup")
-            logger.success(f"Server started http://{address[0]}:{address[1]}")
+            logger.success(
+                f"Server started http://{address[0]}:{address[1]} with superpath '{_sitemap.superpath}'")
             self._server.serve_forever()
         except KeyboardInterrupt:
             self._server.shutdown()
@@ -134,7 +174,7 @@ class StaticSite:
 
 
 class Sitemap:
-    def __init__(self, superpath: str = "/", error_page: StaticSite = None):
+    def __init__(self, superpath: str = "/", error_page: dict = None):
         """
         Create a new Sitemap instance
         :param superpath: path which will replace every occurence of '/SUPERPATH/' or 'SUPERPATH/'. Great for accessing multiple servers from one domain and forwarding the requests to this server.
@@ -150,12 +190,21 @@ class Sitemap:
             "post": {}
         }
         if error_page == None:
-            self.error_page = StaticSite("/error", "none")
+            self.error_page = {0: StaticSite(
+                "/error", "none"), 404: default_sites.page_not_found_error, 500: default_sites.general_server_error}
         elif issubclass(error_page.__class__, StaticSite):
-            self.error_page = error_page
+            self.error_page = {0: error_page}
+        elif type(error_page) == dict:
+            for key, value in error_page.items():
+                if type(key) != int:
+                    raise TypeError(
+                        "error_page: dict keys not of type int (are used as response_codes)")
+                if not issubclass(error_page.__class__, StaticSite) and not callable(error_page):
+                    raise TypeError(
+                        "error_page is neither a StaticSite nor a function.")
         else:
             raise Exception(
-                "error_page argument expected to a be of subclass 'Site'")
+                "error_page argument expected to of type dict[int, Site], or a subclass of 'StaticSite'")
 
     def register_site(self, method: str, site: StaticSite, path=None):
         logger.set_context("registration")
@@ -196,18 +245,7 @@ class Sitemap:
             logger.warning(
                 f"Site for path '{path}' not found. Cannot be unregistered.")
 
-    def get_content(self, method: str, path: str, received_data: str = ""):
-        response_code = 500
-        text = ""
-        info = {}
-        method = get_http_method_type(method)
-        check_relative_path(path)
-        site = None
-        for pattern in self.methods[method]:
-            if re.match(pattern, path):
-                site = self.methods[method][pattern]
-        if site == None:
-            site = self.error_page
+    def get_func_or_site_response(self, site, received_data: str):
         if isinstance(site, StaticSite):
             text = site.get_content()
             info = {"Content-type": "text/html",
@@ -260,14 +298,35 @@ class Sitemap:
                 text = ""
         text = text.replace(
             "/SUPERPATH/", self.superpath).replace("SUPERPATH/", self.superpath)
+        return info, text
+
+    def get_content(self, method: str, path: str, received_data: str = ""):
+        response_code = 500
+        text = ""
+        info = {}
+        method = get_http_method_type(method)
+        check_relative_path(path)
+        site = None
+        for pattern in self.methods[method]:
+            if re.match(pattern, path):
+                site = self.methods[method][pattern]
+        if site == None:
+            response_code = 404
+            site = self.error_page.get(404, self.error_page[0])
+        try:
+            info, text = self.get_func_or_site_response(site, received_data)
+        except Exception as e:
+            logger.handle_exception(e)
+            site = self.error_page.get(500, self.error_page[0])
+            info, text = self.get_func_or_site_response(site, "")
         return response_code, text, info
 
 
 _sitemap = Sitemap()
 
 
-def serves_get(path):
-    """when using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
+def serves_get(path: str):
+    """When using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
 
     Example(s): 
 
@@ -290,8 +349,8 @@ def serves_get(path):
     return my_wrap
 
 
-def serves_post(path):
-    """when using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
+def serves_post(path: str):
+    """When using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
 
     Example(s): 
 
@@ -315,7 +374,7 @@ def serves_post(path):
 
 
 def static_page(file_path, path):
-    """register static page"""
+    """Register a static page while the file is located under `file_path` and will serve `path`"""
     check_relative_file_path(file_path)
     check_relative_path(path)
     site = StaticSite(path, file_path)
@@ -323,27 +382,28 @@ def static_page(file_path, path):
 
 
 def register_get(func, path: str):
-    """register dynamic GET page (function)"""
+    """Register dynamic GET page (function)"""
     check_relative_path(path)
     if callable(func):
         _sitemap.register_site("GET", func, path)
 
 
 def register_post(func, path: str):
-    """register dynamic POST page (function)"""
+    """Register dynamic POST page (function)"""
     check_relative_path(path)
     if callable(func):
         _sitemap.register_site("POST", func, path)
 
 
 def unregister(method: str, path: str):
-    """unregister any page (static or dynamic)."""
+    """Unregister any page (static or dynamic). Only affect the `method`-path (GET / POST)"""
     check_relative_path(path)
     method = get_http_method_type(method)
     _sitemap.unregister_site(method, path)
 
 
-def start(superpath="/"):
+def start(superpath: str = "/"):
+    """Start the server after applying all relevant attributes like address. `superpath` will replace every occurence of SUPERPATH/ or /SUPERPATH/ with `superpath`. Especially useful for servers orchestrating other servers."""
     logger.autosave = True
     logger.header(True, True, description, fileloghelper_version=True,
                   program_version="serverly v" + version)
